@@ -1,18 +1,21 @@
 #!/usr/bin/env python
-
+# -*- coding: utf-8 -*-
+"""
+This module contains methods for exporting MusicXML files.
+"""
 import math
 from collections import defaultdict
 from lxml import etree
-import logging
 import partitura.score as score
 from operator import itemgetter
+from typing import Optional
 
 from .importmusicxml import DYN_DIRECTIONS, PEDAL_DIRECTIONS
 from partitura.utils import partition, iter_current_next, to_quarter_tempo
 
-__all__ = ["save_musicxml"]
+from partitura.utils.misc import deprecated_alias, PathLike
 
-LOGGER = logging.getLogger(__name__)
+__all__ = ["save_musicxml"]
 
 DOCTYPE = """<!DOCTYPE score-partwise PUBLIC\n  "-//Recordare//DTD MusicXML 3.1 Partwise//EN"\n  "http://www.musicxml.org/dtds/partwise.dtd">"""  # noqa: E501
 MEASURE_SEP_COMMENT = "======================================================="
@@ -60,7 +63,7 @@ def filter_string(s):
     return s.replace("\x00", "")
 
 
-def make_note_el(note, dur, voice, counter):
+def make_note_el(note, dur, voice, counter, n_of_staves):
     # child order
     # <grace> | <chord> | <cue>
     # <pitch>
@@ -106,9 +109,25 @@ def make_note_el(note, dur, voice, counter):
 
         etree.SubElement(pitch_e, "octave").text = "{}".format(note.octave)
 
-    elif isinstance(note, score.Rest):
+    elif isinstance(note, score.UnpitchedNote):
 
-        etree.SubElement(note_e, "rest")
+        unpitch_e = etree.SubElement(note_e, "unpitched")
+
+        etree.SubElement(unpitch_e, "display-step").text = "{}".format(note.step)
+
+        etree.SubElement(unpitch_e, "display-octave").text = "{}".format(note.octave)
+
+        if note.notehead is not None:
+            nh_e = etree.SubElement(note_e, "notehead")
+            nh_e.text = "{}".format(note.notehead)
+            if note.noteheadstyle:
+                nh_e.attrib["filled"] = "yes"
+            else:
+                nh_e.attrib["filled"] = "no"
+
+    elif isinstance(note, score.Rest):
+        if not note.hidden:
+            etree.SubElement(note_e, "rest")
 
     if not isinstance(note, score.GraceNote):
 
@@ -165,9 +184,9 @@ def make_note_el(note, dur, voice, counter):
         normal_e = etree.SubElement(time_mod_e, "normal-notes")
         normal_e.text = str(sym_dur["normal_notes"])
 
-    if note.staff:
-
-        etree.SubElement(note_e, "staff").text = "{}".format(note.staff)
+    if note.staff is not None:
+        if note.staff != 1 or n_of_staves > 1:
+            etree.SubElement(note_e, "staff").text = "{}".format(note.staff)
 
     for slur in note.slur_stops:
 
@@ -225,7 +244,7 @@ def make_note_el(note, dur, voice, counter):
     return note_e
 
 
-def do_note(note, measure_end, timeline, voice, counter):
+def do_note(note, measure_end, part, voice, counter, n_of_staves):
     if isinstance(note, score.GraceNote):
 
         dur_divs = 0
@@ -234,7 +253,7 @@ def do_note(note, measure_end, timeline, voice, counter):
 
         dur_divs = note.end.t - note.start.t
 
-    note_e = make_note_el(note, dur_divs, voice, counter)
+    note_e = make_note_el(note, dur_divs, voice, counter, n_of_staves)
 
     return (note.start.t, dur_divs, note_e)
 
@@ -438,6 +457,8 @@ def linearize_segment_contents(part, start, end, state):
         # voice_notes.sort(key=lambda n: -n.duration)
         voice_notes.sort(key=lambda n: n.start.t)
 
+        n_of_staves = part.number_of_staves
+
         for n in voice_notes:
             if isinstance(n, score.GraceNote):
                 # check if it is the first in its sequence
@@ -446,11 +467,13 @@ def linearize_segment_contents(part, start, end, state):
                     # the correct order
                     for m in n.iter_grace_seq():
                         note_e = do_note(
-                            m, end.t, part, voice, state["note_id_counter"]
+                            m, end.t, part, voice, state["note_id_counter"], n_of_staves
                         )
                         voices_e[voice].append(note_e)
             else:
-                note_e = do_note(n, end.t, part, voice, state["note_id_counter"])
+                note_e = do_note(
+                    n, end.t, part, voice, state["note_id_counter"], n_of_staves
+                )
                 voices_e[voice].append(note_e)
 
         add_chord_tags(voices_e[voice])
@@ -775,8 +798,9 @@ def do_directions(part, start, end, counter):
                     # For Flake8 (ignore unused variable), since
                     # etree.SubElement adds e2s to e1s
                     e2s = etree.SubElement(  # noqa: F841
-                        e1s, "pedal", type="start", **pedal_kwargs)
-                if direction.staff is not None:
+                        e1s, "pedal", type="start", **pedal_kwargs
+                    )
+                if direction.staff is not None and direction.staff != 1:
                     e3s = etree.SubElement(e0s, "staff")
                     e3s.text = str(direction.staff)
                 elem = (direction.start.t, None, e0s)
@@ -793,8 +817,9 @@ def do_directions(part, start, end, counter):
                     # For Flake8 (ignore unused variable), since
                     # etree.SubElement adds e2e to e1e
                     e2e = etree.SubElement(  # noqa: F841
-                        e1e, "pedal", type="end", **pedal_kwargs)
-                if direction.staff is not None:
+                        e1e, "pedal", type="end", **pedal_kwargs
+                    )
+                if direction.staff is not None and direction.staff != 1:
                     e3e = etree.SubElement(e0e, "staff")
                     e3e.text = str(direction.staff)
                 elem = (ped_end.t, None, e0e)
@@ -835,7 +860,7 @@ def do_directions(part, start, end, counter):
                         e3, "dashes", number="{}".format(number), type="start"
                     )
 
-            if direction.staff is not None:
+            if direction.staff is not None and direction.staff != 1:
 
                 e5 = etree.SubElement(e0, "staff")
                 e5.text = str(direction.staff)
@@ -923,9 +948,9 @@ def do_attributes(part, start, end):
 
                 clef_e = etree.SubElement(attr_e, "clef")
 
-                if o.number:
+                if o.staff and o.staff != 1:
 
-                    clef_e.set("number", "{}".format(o.number))
+                    clef_e.set("number", "{}".format(o.staff))
 
                 etree.SubElement(clef_e, "sign").text = "{}".format(o.sign)
                 etree.SubElement(clef_e, "line").text = "{}".format(o.line)
@@ -941,14 +966,20 @@ def do_attributes(part, start, end):
     return result
 
 
-def save_musicxml(parts, out=None):
-    """Save a one or more Part or PartGroup instances in MusicXML format.
+@deprecated_alias(parts="score_data")
+def save_musicxml(
+        score_data: score.ScoreLike,
+        out: Optional[PathLike] = None,
+) -> Optional[str]:
+    """
+    Save a one or more Part or PartGroup instances in MusicXML format.
 
     Parameters
     ----------
-    parts : list, Part, or PartGroup
-        A :class:`partitura.score.Part` object,
-        :class:`partitura.score.PartGroup` or a list of these
+    score_data : Score, list, Part, or PartGroup
+        The musical score to be saved. A :class:`partitura.score.Score` object,
+        a :class:`partitura.score.Part`, a :class:`partitura.score.PartGroup` or
+        a list of these.
     out: str, file-like object, or None, optional
         Output file
 
@@ -957,9 +988,14 @@ def save_musicxml(parts, out=None):
     None or str
         If no output file is specified using `out` the function returns the
         MusicXML data as a string. Otherwise the function returns None.
-
     """
 
+    if not isinstance(score_data, score.Score):
+        score_data = score.Score(
+            id=None,
+            partlist=score_data,
+        )
+
     root = etree.Element("score-partwise")
 
     partlist_e = etree.SubElement(root, "part-list")
@@ -1023,7 +1059,7 @@ def save_musicxml(parts, out=None):
 
             group_stack.append(pg)
 
-    for part in score.iter_parts(parts):
+    for part in score_data:
 
         handle_parents(part)
 
