@@ -1,8 +1,14 @@
 #!/usr/bin/env python
-import numpy as np
-from collections import defaultdict
+# -*- coding: utf-8 -*-
+"""
+This module contains methods for importing MIDI files.
+"""
 import warnings
-import logging
+
+from collections import defaultdict
+from typing import Union, Optional
+import numpy as np
+
 
 import mido
 
@@ -13,21 +19,25 @@ from partitura.utils import (
     key_name_to_fifths_mode,
     fifths_mode_to_key_name,
     estimate_clef_properties,
+    deprecated_alias,
+    deprecated_parameter,
+    PathLike,
+    get_document_name,
+    ensure_notearray
 )
 import partitura.musicanalysis as analysis
 
-__all__ = ["load_score_midi", "load_performance_midi"]
-
-LOGGER = logging.getLogger(__name__)
+__all__ = ["load_score_midi", "load_performance_midi", "midi_to_notearray"]
 
 
 # as key for the dict use channel * 128 (max number of pitches) + pitch
-def note_hash(channel, pitch):
+def note_hash(channel: int, pitch: int) -> int:
     """Generate a note hash."""
     return channel * 128 + pitch
 
 
-def midi_to_notearray(fn):
+@deprecated_alias(fn="filename")
+def midi_to_notearray(filename: PathLike) -> np.ndarray:
     """Load a MIDI file in a note_array.
 
     This function should be used to load MIDI files into an
@@ -39,7 +49,7 @@ def midi_to_notearray(fn):
 
     Parameters
     ----------
-    fn : str
+    filename : str
         Path to MIDI file
     Returns
     -------
@@ -47,13 +57,22 @@ def midi_to_notearray(fn):
         Structured array with onset, duration, pitch, velocity, and
         ID fields.
     """
-    ppart = load_performance_midi(fn, merge_tracks=True)
+    perf = load_performance_midi(filename, merge_tracks=True)
     # set sustain pedal threshold to 128 to disable sustain adjusted offsets
-    ppart.sustain_pedal_threshold = 128
-    return ppart.note_array
+
+    for ppart in perf:
+        ppart.sustain_pedal_threshold = 128
+
+    note_array = ensure_notearray(perf)
+    return note_array
 
 
-def load_performance_midi(fn, default_bpm=120, merge_tracks=False):
+@deprecated_alias(fn="filename")
+def load_performance_midi(
+    filename: Union[PathLike, mido.MidiFile],
+    default_bpm: Union[int, float] = 120,
+    merge_tracks: bool = False
+) -> performance.Performance:
     """Load a musical performance from a MIDI file.
 
     This function should be used for MIDI files that encode
@@ -68,20 +87,27 @@ def load_performance_midi(fn, default_bpm=120, merge_tracks=False):
 
     Parameters
     ----------
-    fn : str
+    filename : str
         Path to MIDI file
     default_bpm : number, optional
         Tempo to use wherever the MIDI does not specify a tempo.
         Defaults to 120.
+    merge_tracks: bool, optional
+        For MIDI files, merges all tracks into a single track.
 
     Returns
     -------
-    :class:`partitura.performance.PerformedPart`
-        A PerformedPart instance.
-
-
+    :class:`partitura.performance.Performance`
+        A Performance instance.
     """
-    mid = mido.MidiFile(fn)
+
+    if isinstance(filename, mido.MidiFile):
+        mid = filename
+        doc_name = filename.filename
+    else:
+        mid = mido.MidiFile(filename)
+        doc_name = get_document_name(filename)
+
     # parts per quarter
     ppq = mid.ticks_per_beat
     # microseconds per quarter
@@ -90,43 +116,41 @@ def load_performance_midi(fn, default_bpm=120, merge_tracks=False):
     # convert MIDI ticks in seconds
     time_conversion_factor = mpq / (ppq * 10 ** 6)
 
-    notes = []
-    controls = []
-    programs = []
+    pps = list()
+    
     if merge_tracks:
         mid_merge = mido.merge_tracks(mid.tracks)
         tracks = [(0, mid_merge)]
     else:
         tracks = [(i, u) for i, u in enumerate(mid.tracks)]
     for i, track in tracks:
+        
+        notes = []
+        controls = []
+        programs = []
 
         t = 0
+        ttick = 0
+        
         sounding_notes = {}
 
         for msg in track:
 
             # update time deltas when they arrive
             t = t + msg.time * time_conversion_factor
+            ttick = ttick + msg.time 
 
             if msg.type == "set_tempo":
 
                 mpq = msg.tempo
-
                 time_conversion_factor = mpq / (ppq * 10 ** 6)
 
-                LOGGER.info(
-                    (
-                        "change of Tempo to mpq = {0} "
-                        " and resulting seconds per tick = {1}"
-                        "at time: {2}"
-                    ).format(mpq, time_conversion_factor, t)
-                )
-
             elif msg.type == "control_change":
 
                 controls.append(
                     dict(
                         time=t,
+                        time_tick=ttick,
                         number=msg.control,
                         value=msg.value,
                         track=i,
@@ -137,7 +161,13 @@ def load_performance_midi(fn, default_bpm=120, merge_tracks=False):
             elif msg.type == "program_change":
 
                 programs.append(
-                    dict(time=t, program=msg.program, track=i, channel=msg.channel)
+                    dict(
+                        time=t,
+                        time_tick=ttick,
+                        program=msg.program,
+                        track=i,
+                        channel=msg.channel,
+                    )
                 )
 
             else:
@@ -155,7 +185,7 @@ def load_performance_midi(fn, default_bpm=120, merge_tracks=False):
                 if note_on and msg.velocity > 0:
 
                     # save the onset time and velocity
-                    sounding_notes[note] = (t, msg.velocity)
+                    sounding_notes[note] = (t, ttick, msg.velocity)
 
                 # end note if it's a 'note off' event or 'note on' with velocity 0
                 elif note_off or (note_on and msg.velocity == 0):
@@ -168,31 +198,63 @@ def load_performance_midi(fn, default_bpm=120, merge_tracks=False):
 
                     notes.append(
                         dict(
-                            id=f"n{len(notes)}",
+                            # id=f"n{len(notes)}",
                             midi_pitch=msg.note,
                             note_on=(sounding_notes[note][0]),
+                            note_on_tick=(sounding_notes[note][1]),
                             note_off=(t),
+                            note_off_tick=(ttick),
                             track=i,
                             channel=msg.channel,
-                            velocity=sounding_notes[note][1],
+                            velocity=sounding_notes[note][2],
                         )
                     )
-
                     # remove hash from dict
                     del sounding_notes[note]
 
-    return performance.PerformedPart(notes, controls=controls, programs=programs)
+        # fix note ids so that it is sorted lexicographically
+        # by onset, pitch, offset, channel and track
+        notes.sort(
+            key=lambda x: (
+                x["note_on"],
+                x["midi_pitch"],
+                x["note_off"],
+                x["channel"],
+                x["track"],
+            )
+        )
+
+        # add note id to every note
+        for k, note in enumerate(notes):
+            note["id"] = f"n{k}"
+            
+        if len(notes) > 0 or len(controls) > 0 or len(programs) > 0:
+            pp = performance.PerformedPart(notes, 
+                                    controls=controls, 
+                                    programs=programs, 
+                                    ppq = ppq,
+                                    mpq = mpq,
+                                    track = i)
+            
+            pps.append(pp)
+
+    perf = performance.Performance(
+        id=doc_name,
+        performedparts=pps,
+    )
+    return perf
 
 
+@deprecated_parameter("ensure_list")
+@deprecated_alias(fn="filename")
 def load_score_midi(
-    fn,
-    part_voice_assign_mode=0,
-    ensure_list=False,
-    quantization_unit=None,
-    estimate_voice_info=True,
-    estimate_key=False,
-    assign_note_ids=True,
-):
+    filename: Union[PathLike, mido.MidiFile],
+    part_voice_assign_mode: Optional[int] = 0,
+    quantization_unit: Optional[int] = None,
+    estimate_voice_info: bool = True,
+    estimate_key: bool = False,
+    assign_note_ids: bool = True,
+) -> score.Score:
     """Load a musical score from a MIDI file and return it as a Part
     instance.
 
@@ -211,8 +273,8 @@ def load_score_midi(
 
     Parameters
     ----------
-    fn : str
-        Path to MIDI file
+    filename : PathLike or mido.MidiFile
+        Path to MIDI file or mido.MidiFile object.
     part_voice_assign_mode : {0, 1, 2, 3, 4, 5}, optional
         This keyword controls how part and voice information is
         associated to track and channel information in the MIDI file.
@@ -235,13 +297,6 @@ def load_score_midi(
         5
             Return one Part per <track, channel> combination, without
             voices  Defaults to 0.
-    ensure_list : bool, optional
-        When True, return a list independent of how many part or partgroup
-        elements were created from the MIDI file. By default, when the
-        return value of `load_score_midi` produces a single
-        :class:`partitura.score.Part` or :class:`partitura.score.PartGroup`
-        element, the element itself is returned instead of a list
-        containing the element. Defaults to False.
     quantization_unit : integer or None, optional
         Quantize MIDI times to multiples of this unit. If None, the
         quantization unit is chosen automatically as the smallest
@@ -276,7 +331,14 @@ or a list of these
            Oxford University Press, New York.
 
     """
-    mid = mido.MidiFile(fn)
+
+    if isinstance(filename, mido.MidiFile):
+        mid = filename
+        doc_name = filename.filename
+    else:
+        mid = mido.MidiFile(filename)
+        doc_name = get_document_name(filename)
+
     divs = mid.ticks_per_beat
 
     # these lists will contain information from dedicated tracks for meta
@@ -401,11 +463,11 @@ or a list of these
         dtype=[("onset_div", int), ("pitch", int), ("duration_div", int)],
     )
 
-    LOGGER.debug("pitch spelling")
+    warnings.warn("pitch spelling")
     spelling_global = analysis.estimate_spelling(note_array)
 
     if estimate_voice_info:
-        LOGGER.debug("voice estimation")
+        warnings.warn("voice estimation", stacklevel=2)
         # TODO: deal with zero duration notes in note_array.
         # Zero duration notes are currently deleted
         estimated_voices = analysis.estimate_voices(note_array)
@@ -415,7 +477,7 @@ or a list of these
                 part_voice[1] = voice_est
 
     if estimate_key:
-        LOGGER.debug("key estimation")
+        warnings.warn("key estimation", stacklevel=2)
         _, mode, fifths = analysis.estimate_key(note_array)
         key_sigs_by_track = {}
         global_key_sigs = [(0, fifths_mode_to_key_name(fifths, mode))]
@@ -494,10 +556,13 @@ or a list of these
     for t, qpm in global_tempos:
         part.add(score.Tempo(qpm, unit="q"), t)
 
-    if not ensure_list and len(partlist) == 1:
-        return partlist[0]
-    else:
-        return partlist
+    # TODO: Add info (composer, etc.)
+    scr = score.Score(
+        id=doc_name,
+        partlist=partlist,
+    )
+
+    return scr
 
 
 def make_track_to_part_mapping(tr_ch_keys, group_part_voice_keys):
@@ -584,21 +649,21 @@ def create_part(
     key_sigs,
     part_id=None,
     part_name=None,
-):
-    LOGGER.debug("create_part")
+) -> score.Part:
+    warnings.warn("create_part", stacklevel=2)
 
     part = score.Part(part_id, part_name=part_name)
     part.set_quarter_duration(0, ticks)
 
     clef = score.Clef(
-        number=1, **estimate_clef_properties([pitch for _, pitch, _ in notes])
+        staff=1, **estimate_clef_properties([pitch for _, pitch, _ in notes])
     )
     part.add(clef, 0)
     for t, name in key_sigs:
         fifths, mode = key_name_to_fifths_mode(name)
         part.add(score.KeySignature(fifths, mode), t)
 
-    LOGGER.debug("add notes")
+    warnings.warn("add notes", stacklevel=2)
 
     for (onset, pitch, duration), (step, alter, octave), voice, note_id in zip(
         notes, spellings, voices, note_ids
@@ -635,7 +700,7 @@ def create_part(
     ts_end_times = np.r_[time_sigs[1:, 0], np.iinfo(int).max]
     time_sigs = np.column_stack((time_sigs, ts_end_times))
 
-    LOGGER.debug("add time sigs and measures")
+    warnings.warn("add time sigs and measures", stacklevel=2)
 
     for ts_start, num, den, ts_end in time_sigs:
         time_sig = score.TimeSignature(num.item(), den.item())
@@ -662,16 +727,16 @@ def create_part(
     #     if np.isinf(ts_end):
     #         ts_end = m_end
 
-    LOGGER.debug("tie notes")
+    warnings.warn("tie notes", stacklevel=2)
     # tie notes where necessary (across measure boundaries, and within measures
     # notes with compound duration)
     score.tie_notes(part)
 
-    LOGGER.debug("find tuplets")
+    warnings.warn("find tuplets", stacklevel=2)
     # apply simplistic tuplet finding heuristic
     score.find_tuplets(part)
 
-    LOGGER.debug("done create_part")
+    warnings.warn("done create_part", stacklevel=2)
     return part
 
 
