@@ -1,23 +1,30 @@
 #!/usr/bin/env python
-
 # -*- coding: utf-8 -*-
+"""
+This module contains methods for importing MusicXML files.
+"""
 
-import logging
+import os
+import warnings
+import zipfile
 
+from typing import Union, Optional
 import numpy as np
 from lxml import etree
 
+
 # lxml does XSD validation too but has problems with the MusicXML 3.1 XSD, so we use
 # the xmlschema package for validating MusicXML against the definition
 import xmlschema
 import pkg_resources
 from partitura.directions import parse_direction
 import partitura.score as score
+from partitura.score import assign_note_ids
 from partitura.utils import ensure_notearray
+from partitura.utils.misc import deprecated_alias, deprecated_parameter, PathLike
 
-__all__ = ["load_musicxml"]
+__all__ = ["load_musicxml", "musicxml_to_notearray"]
 
-LOGGER = logging.getLogger(__name__)
 _MUSICXML_SCHEMA = pkg_resources.resource_filename("partitura", "assets/musicxml.xsd")
 _XML_VALIDATOR = None
 DYN_DIRECTIONS = {
@@ -148,13 +155,19 @@ def _parse_partlist(partlist):
                 part.parent = current_group
 
     if current_group is not None:
-        LOGGER.warning("part-group {0} was not ended".format(current_group.number))
+        warnings.warn("part-group {0} was not ended".format(current_group.number))
         structure.append(current_group)
 
     return structure, part_dict
 
 
-def load_musicxml(xml, ensure_list=False, validate=False, force_note_ids=None):
+@deprecated_alias(xml="filename")
+@deprecated_parameter("ensure_list")
+def load_musicxml(
+        filename: PathLike,
+        validate: bool = False,
+        force_note_ids: Optional[Union[bool, str]] = None
+) -> score.Score:
     """Parse a MusicXML file and build a composite score ontology
     structure from it (see also scoreontology.py).
 
@@ -162,14 +175,6 @@ def load_musicxml(xml, ensure_list=False, validate=False, force_note_ids=None):
     ----------
     xml : str or file-like  object
         Path to the MusicXML file to be parsed, or a file-like object
-    ensure_list : bool, optional
-        When True return a list independent of how many part or
-        partgroup elements were created from the MIDI file. By
-        default, when the return value of `load_musicxml` produces a
-    single : class:`partitura.score.Part` or
-        :Class:`partitura.score.PartGroup` element, the element itself
-        is returned instead of a list containing the element. Defaults
-        to False.
     validate : bool, optional
         When True the validity of the MusicXML is checked against the
         MusicXML 3.1 specification before loading the file. An
@@ -183,10 +188,22 @@ def load_musicxml(xml, ensure_list=False, validate=False, force_note_ids=None):
 
     Returns
     -------
-    partlist : list
-        A list of either Part or PartGroup objects
+    scr: Score
+        A `Score` instance.
 
     """
+
+    xml = None
+    if isinstance(filename, str):
+        if zipfile.is_zipfile(filename):
+            with zipfile.ZipFile(filename) as zipped_xml:
+                xml = zipped_xml.open(
+                    os.path.splitext(os.path.basename(filename))[0] + ".xml"
+                )
+
+    if xml is None:
+        xml = filename
+
     if validate:
         validate_musicxml(xml, debug=True)
         # if xml is a file-like object we need to set the read pointer to the
@@ -220,37 +237,68 @@ def load_musicxml(xml, ensure_list=False, validate=False, force_note_ids=None):
     if force_note_ids is True or force_note_ids == "keep":
         assign_note_ids(partlist, force_note_ids == "keep")
 
-    if not ensure_list and len(partlist) == 1:
-        return partlist[0]
-    else:
-        return partlist
+    composer = None
+    scid = None
+    title = None
+    subtitle = None
+    lyricist = None
+    copyright = None
 
+    # The work tag is preferred for the title of the score, otherwise
+    # this method will search in the credit tags
+    work_info_el = document.find("work")
 
-def assign_note_ids(parts, keep=False):
-    if keep:
-        # Keep existing note id's
-        for p, part in enumerate(score.iter_parts(parts)):
-            for ni, n in enumerate(
-                part.iter_all(score.GenericNote, include_subclasses=True)
-            ):
-                if isinstance(n, score.Rest):
-                    n.id = "p{0}r{1}".format(p, ni) if n.id is None else n.id
-                else:
-                    n.id = "p{0}n{1}".format(p, ni) if n.id is None else n.id
+    if work_info_el is not None:
+        scid = get_value_from_tag(
+            e=work_info_el,
+            tag="work-title",
+            as_type=str,
+        )
 
-    else:
-        # assign note ids to ensure uniqueness across all parts, discarding any
-        # existing note ids
-        ni = 0
-        ri = 0
-        for part in score.iter_parts(parts):
-            for n in part.iter_all(score.GenericNote, include_subclasses=True):
-                if isinstance(n, score.Rest):
-                    n.id = "r{}".format(ri)
-                    ri += 1
-                else:
-                    n.id = "n{}".format(ni)
-                    ni += 1
+        title = scid
+
+    score_identification_el = document.find("identification")
+
+    # The identification tag has preference over credit
+    if score_identification_el is not None:
+
+        copyright = get_value_from_tag(score_identification_el, "rights", str)
+
+        for cel in score_identification_el.findall("creator"):
+            if get_value_from_attribute(cel, "type", str) == "composer":
+                composer = str(cel.text)
+            if get_value_from_attribute(cel, "type", str) == "lyricist":
+                lyricist = str(cel.text)
+
+    for cel in document.findall("credit"):
+
+        credit_type = get_value_from_tag(cel, "credit-type", str)
+        if credit_type == "title" and title is None:
+            title = get_value_from_tag(cel, "credit-words", str)
+
+        elif credit_type == "subtitle" and subtitle is None:
+            subtitle = get_value_from_tag(cel, "credit-words", str)
+
+        elif credit_type == "composer" and composer is None:
+            composer = get_value_from_tag(cel, "credit-words", str)
+
+        elif credit_type == "lyricist" and lyricist is None:
+            lyricist = get_value_from_tag(cel, "credit-words", str)
+
+        elif credit_type == "rights" and copyright is None:
+            copyright = get_value_from_tag(cel, "credit-words", str)
+
+    scr = score.Score(
+        id=scid,
+        partlist=partlist,
+        title=title,
+        subtitle=subtitle,
+        composer=composer,
+        lyricist=lyricist,
+        copyright=copyright,
+    )
+
+    return scr
 
 
 def _parse_parts(document, part_dict):
@@ -284,9 +332,77 @@ def _parse_parts(document, part_dict):
                 measure_el, position, part, ongoing, doc_order
             )
 
+        # complete unfinished endings
+        for o in part.iter_all(score.Ending, mode="ending"):
+            if o.start is None:
+                part.add(o, part.measure_map(o.end.t - 1)[0], None)
+                warnings.warn(
+                    "Found ending[stop] without a preceding ending[start]\n"
+                    "Single measure bracket is assumed"
+                )
+
+        for o in part.iter_all(score.Ending, mode="starting"):
+            if o.end is None:
+                part.add(o, None, part.measure_map(o.start.t)[1])
+                warnings.warn(
+                    "Found ending[start] without a following ending[stop]\n"
+                    "Single measure bracket is assumed"
+                )
+
+        # Complete repeats without end.
+        volta_repeats = list()
+        for o in part.iter_all(score.Repeat, mode="starting"):
+            if o.end is None:
+                # if len(o.start.starting_objects[score.Repeat]) > 0:
+                #     starting = list(o.start.starting_objects[score.Repeat].keys())[0]
+                #     # if unstarted repeat from volta, continue for now
+                #     if len(starting.end.ending_objects[score.Repeat]) > 0:
+                #         # if repeat from volta, continue for now
+                #         volta_repeats.append(o)
+                #         continue
+                
+                starting_repeats = [r for r in part.iter_all(score.Repeat) if r.start is not None] 
+                end_times = [r.start.t for r in starting_repeats] + [part._points[-1].t]
+                end_time_id = np.searchsorted(end_times, o.start.t+1)
+                part.add(o, None, end_times[end_time_id])
+                warnings.warn(
+                    "Found repeat without end\n"
+                    "Ending point {} is assumed".format(end_times[end_time_id])
+                )
+
+        # complete unstarted repeats
+        volta_repeats = list()
+        for o in part.iter_all(score.Repeat, mode="ending"):
+            if o.start is None:
+                if len(o.end.ending_objects[score.Ending]) > 0:
+                    ending = list(o.end.ending_objects[score.Ending].keys())[0]
+                    # if unstarted repeat from volta, continue for now
+                    if len(ending.start.ending_objects[score.Repeat]) > 0:
+                        volta_repeats.append(o)
+                        continue
+
+                # go back to the end of the last repeat
+                start_times = [0] + [r.end.t for r in part.iter_all(score.Repeat)]
+                start_time_id = np.searchsorted(start_times, o.end.t) - 1
+                part.add(o, start_times[start_time_id], None)
+                warnings.warn(
+                    "Found repeat without start\n"
+                    "Starting point {} is assumed".format(start_times[start_time_id])
+                )
+
+        # complete unstarted repeats in volta with start time of first repeat
+        for o in volta_repeats:
+            start_times = [0] + [r.start.t for r in part.iter_all(score.Repeat)]
+            start_time_id = np.searchsorted(start_times, o.end.t) - 1
+            part.add(o, start_times[start_time_id], None)
+            warnings.warn(
+                "Found repeat without start\n"
+                "Starting point {} is assumed".format(start_times[start_time_id])
+            )
+
         # remove unfinished elements from the timeline
         for k, o in ongoing.items():
-            if k not in ("page", "system"):
+            if k not in ("page", "system", "repeat") and k[0] not in ("tie", "ending"):
                 if isinstance(o, list):
                     for o_i in o:
                         part.remove(o_i)
@@ -307,10 +423,10 @@ def _parse_parts(document, part_dict):
                         gn.last_grace_note_in_seq.grace_next = no
 
             if gn.main_note is None:
-                print(
+                warnings.warn(
                     "grace note without recoverable same voice main note: {}".format(gn)
                 )
-                print("might be cadenza notation")
+                warnings.warn("might be cadenza notation")
 
         # set end times for various musical elements that only have a start time
         # when constructed from MusicXML
@@ -351,11 +467,11 @@ def _handle_measure(measure_el, position, part, ongoing, doc_order):
             position -= duration
 
             if position < measure.start.t:
-                LOGGER.warning(
-                    ("<backup> crosses measure boundary, adjusting "
-                     "position from {} to {} in Measure {}").format(
-                        position, measure.start.t, measure.number
-                    )
+                warnings.warn(
+                    (
+                        "<backup> crosses measure boundary, adjusting "
+                        "position from {} to {} in Measure {}"
+                    ).format(position, measure.start.t, measure.number)
                 )
                 position = measure.start.t
 
@@ -394,14 +510,21 @@ def _handle_measure(measure_el, position, part, ongoing, doc_order):
             measure_maxtime = max(measure_maxtime, position)
 
         elif e.tag == "barline":
+            location = get_value_from_attribute(e, "location", str)
+            if (location is None) or (location == "right"):
+                position_barline = measure_maxtime
+            elif location == "left":
+                position_barline = measure_start
+            else:
+                position_barline = position
 
             repeat = e.find("repeat")
             if repeat is not None:
-                _handle_repeat(repeat, position, part, ongoing)
+                _handle_repeat(repeat, position_barline, part, ongoing)
 
             ending = e.find("ending")
             if ending is not None:
-                _handle_ending(ending, position, part, ongoing)
+                _handle_ending(ending, position_barline, part, ongoing)
 
             bar_style_e = e.find("bar-style")
             if bar_style_e is not None:
@@ -431,7 +554,7 @@ def _handle_measure(measure_el, position, part, ongoing, doc_order):
             # TODO: handle segno/fine/dacapo
 
         else:
-            LOGGER.debug("ignoring tag {0}".format(e.tag))
+            warnings.warn("ignoring tag {0}".format(e.tag), stacklevel=2)
 
     for obj in trailing_children:
         part.add(obj, measure_maxtime)
@@ -460,13 +583,15 @@ def _handle_repeat(e, position, part, ongoing):
             # object and add it at the beginning of
             # the self retroactively
             o = score.Repeat()
-            part.add(o, 0)
+            # no, clean up later with shortest possible repeat start
+            # part.add(o, 0)
 
         part.add(o, None, position)
 
 
 def _handle_ending(e, position, part, ongoing):
-    key = "ending"
+    # key = "ending"
+    key = ("ending", getattr(e, "number", "0"))
 
     if e.get("type") == "start":
 
@@ -480,7 +605,12 @@ def _handle_ending(e, position, part, ongoing):
 
         if o is None:
 
-            LOGGER.warning("Found ending[stop] without a preceding ending[start]")
+            warnings.warn(
+                "Found ending[stop] without a preceding ending[start]\n"
+                + "Single measure bracket is assumed"
+            )
+            o = score.Ending(e.get("number"))
+            part.add(o, None, position)
 
         else:
 
@@ -595,10 +725,21 @@ def _handle_direction(e, position, part, ongoing):
 
     staff = get_value_from_tag(e, "staff", int) or None
 
-    if get_value_from_attribute(e, "sound/fine", str) == "yes":
-        part.add(score.Fine(), position)
-    if get_value_from_attribute(e, "sound/dacapo", str) == "yes":
-        part.add(score.DaCapo(), position)
+    sound_directions = e.findall("sound")
+    for sd in sound_directions:
+
+        if get_value_from_attribute(sd, "fine", str) == "yes":
+            part.add(score.Fine(), position)
+        if get_value_from_attribute(sd, "dacapo", str) == "yes":
+            part.add(score.DaCapo(), position)
+        if get_value_from_attribute(sd, "tocoda", str) == "coda":
+            part.add(score.ToCoda(), position)
+        if get_value_from_attribute(sd, "coda", str) == "coda":
+            part.add(score.Coda(), position)
+        if get_value_from_attribute(sd, "dalsegno", str) == "segno":
+            part.add(score.DalSegno(), position)
+        if get_value_from_attribute(sd, "segno", str) == "segno":
+            part.add(score.Segno(), position)
 
     # <direction-type> ... </...>
     direction_types = e.findall("direction-type")
@@ -675,7 +816,7 @@ def _handle_direction(e, position, part, ongoing):
                     ending_directions.append(o)
                     del ongoing[key]
                 else:
-                    LOGGER.warning("Did not find a wedge start element for wedge stop!")
+                    warnings.warn("Did not find a wedge start element for wedge stop!")
 
         elif dt.tag == "dashes":
 
@@ -717,17 +858,17 @@ def _handle_direction(e, position, part, ongoing):
                     del ongoing[key]
 
                 else:
-                    LOGGER.warning("Did not find a pedal start element for pedal stop!")
+                    warnings.warn("Did not find a pedal start element for pedal stop!")
 
             else:
                 if pedal_type in ("change", "continue"):
-                    LOGGER.warning(
+                    warnings.warn(
                         'pedal types "change" and "continue" are '
                         "not supported. Ignoring direction."
                     )
 
         else:
-            LOGGER.warning("ignoring direction type: {} {}".format(dt.tag, dt.attrib))
+            warnings.warn("ignoring direction type: {} {}".format(dt.tag, dt.attrib))
 
     for dashes_key, dashes_type in dashes_keys.items():
 
@@ -739,7 +880,7 @@ def _handle_direction(e, position, part, ongoing):
 
             oo = ongoing.get(dashes_key)
             if oo is None:
-                LOGGER.warning("Dashes end without dashes start")
+                warnings.warn("Dashes end without dashes start")
             else:
                 ending_directions.extend(oo)
                 del ongoing[dashes_key]
@@ -769,7 +910,7 @@ def get_clefs(e):
     for clef in clefs:
         result.append(
             dict(
-                number=get_value_from_attribute(clef, "number", int),
+                staff=get_value_from_attribute(clef, "number", int) or 1,
                 sign=get_value_from_tag(clef, "sign", str),
                 line=get_value_from_tag(clef, "line", int),
                 octave_change=get_value_from_tag(clef, "clef-octave-change", int),
@@ -906,7 +1047,7 @@ def _add_tempo_if_unique(position, part, tempo):
         if tempos == []:
             part.add(tempo, position)
         else:
-            LOGGER.warning("not adding duplicate or conflicting tempo indication")
+            warnings.warn("not adding duplicate or conflicting tempo indication")
 
 
 def _handle_sound(e, position, part):
@@ -922,8 +1063,8 @@ def _handle_note(e, position, part, ongoing, prev_note, doc_order):
     duration = get_value_from_tag(e, "duration", int) or 0
     # elements may have an explicit temporal offset
     # offset = get_value_from_tag(e, 'offset', int) or 0
-    staff = get_value_from_tag(e, "staff", int) or None
-    voice = get_value_from_tag(e, "voice", int) or None
+    staff = get_value_from_tag(e, "staff", int) or 1
+    voice = get_value_from_tag(e, "voice", int) or 1
 
     # add support of uppercase "ID" tags
     note_id = (
@@ -962,7 +1103,14 @@ def _handle_note(e, position, part, ongoing, prev_note, doc_order):
     else:
         articulations = {}
 
+    ornaments_e = e.find("notations/ornaments")
+    if ornaments_e is not None:
+        ornaments = get_ornaments(ornaments_e)
+    else:
+        ornaments = {}
+
     pitch = e.find("pitch")
+    unpitch = e.find("unpitched")
     if pitch is not None:
 
         step = get_value_from_tag(pitch, "step", str)
@@ -983,6 +1131,7 @@ def _handle_note(e, position, part, ongoing, prev_note, doc_order):
                 staff=staff,
                 symbolic_duration=symbolic_duration,
                 articulations=articulations,
+                ornaments=ornaments,
                 steal_proportion=steal_proportion,
                 doc_order=doc_order,
             )
@@ -998,11 +1147,39 @@ def _handle_note(e, position, part, ongoing, prev_note, doc_order):
                 staff=staff,
                 symbolic_duration=symbolic_duration,
                 articulations=articulations,
+                ornaments=ornaments,
                 doc_order=doc_order,
             )
 
         if isinstance(prev_note, score.GraceNote) and prev_note.voice == voice:
             prev_note.grace_next = note
+
+    elif unpitch is not None:
+        # note element is unpitched
+        step = get_value_from_tag(unpitch, "display-step", str)
+        octave = get_value_from_tag(unpitch, "display-octave", int)
+        noteheadtag = e.find("notehead")
+        noteheadstylebool = True
+        notehead = None
+        if noteheadtag is not None:
+            notehead = get_value_from_tag(e, "notehead", str)
+            noteheadstyle = get_value_from_attribute(noteheadtag, "filled", str)
+            if noteheadstyle is not None:
+                noteheadstylebool = {"no": False, "yes": True}[noteheadstyle]
+
+        note = score.UnpitchedNote(
+            step=step,
+            octave=octave,
+            id=note_id,
+            voice=voice,
+            staff=staff,
+            notehead=notehead,
+            noteheadstyle=noteheadstylebool,
+            articulations=articulations,
+            symbolic_duration=symbolic_duration,
+            doc_order=doc_order,
+        )
+
     else:
         # note element is a rest
         note = score.Rest(
@@ -1168,9 +1345,9 @@ def handle_slurs(notations, ongoing, note, position):
 
             # if slur.end_note.start.t < position then the slur stop is
             # rogue. We drop it and treat the slur start like a fresh start
-            if slur is None or slur.end_note.start.t <= position:
+            if slur is None or slur.end_note.start.t < position:
 
-                if slur and slur.end_note.start.t <= position:
+                if slur and slur.end_note.start.t < position:
                     msg = (
                         "Dropping slur {} starting at {} ({}) and ending "
                         "at {} ({})".format(
@@ -1181,7 +1358,7 @@ def handle_slurs(notations, ongoing, note, position):
                             slur.end_note.id,
                         )
                     )
-                    LOGGER.warning(msg)
+                    warnings.warn(msg)
                     # remove the slur from the timeline
                     slur.end_note.start.remove_ending_object(slur)
                     # remove the reference to the slur in the end note
@@ -1200,9 +1377,9 @@ def handle_slurs(notations, ongoing, note, position):
 
             slur = ongoing.pop(start_slur_key, None)
 
-            if slur is None or slur.start_note.start.t >= position:
+            if slur is None or slur.start_note.start.t > position:
 
-                if slur and slur.start_note.start.t >= position:
+                if slur and slur.start_note.start.t > position:
                     msg = (
                         "Dropping slur {} starting at {} ({}) and ending "
                         "at {} ({})".format(
@@ -1213,7 +1390,7 @@ def handle_slurs(notations, ongoing, note, position):
                             note.id,
                         )
                     )
-                    LOGGER.warning(msg)
+                    warnings.warn(msg)
                     # remove the slur from the timeline
                     slur.start_note.start.remove_starting_object(slur)
                     # remove the reference to the slur in the end note
@@ -1283,8 +1460,47 @@ def get_articulations(e):
     return [a for a in articulations if e.find(a) is not None]
 
 
+def get_ornaments(e):
+    #  ornaments elements:
+    #  trill-mark
+    #  turn
+    #  delayed-turn
+    #  inverted-turn
+    #  delayed-inverted-turn
+    #  vertical-turn
+    #  inverted-vertical-turn
+    #  shake
+    #  wavy-line
+    #  mordent
+    #  inverted-mordent
+    #  schleifer
+    #  tremolo
+    #  haydn
+    #  other-ornament
+
+    ornaments = (
+        "trill-mark",
+        "turn",
+        "delayed-turn",
+        "inverted-turn",
+        "delayed-inverted-turn",
+        "vertical-turn",
+        "inverted-vertical-turn",
+        "shake",
+        "wavy-line",
+        "mordent",
+        "inverted-mordent",
+        "schleifer",
+        "tremolo",
+        "haydn",
+        "other-ornament",
+    )
+    return [a for a in ornaments if e.find(a) is not None]
+
+
+@deprecated_alias(fn="filename")
 def musicxml_to_notearray(
-    fn,
+    filename,
     flatten_parts=True,
     include_pitch_spelling=False,
     include_key_signature=False,
@@ -1319,15 +1535,18 @@ def musicxml_to_notearray(
 
     Returns
     -------
-    score : structured array or list of structured arrays
+    note_arrays : structured array or list of structured arrays
         Structured array or list of structured arrays containing
         score information.
     """
 
-    parts = load_musicxml(fn, ensure_list=True, force_note_ids="keep")
+    scr = load_musicxml(
+        filename=filename,
+        force_note_ids="keep",
+    )
 
     note_arrays = []
-    for part in score.iter_parts(parts):
+    for part in scr.parts:
         # Unfold any repetitions in part
         unfolded_part = score.unfold_part_maximal(part)
         # Compute note array
@@ -1335,7 +1554,8 @@ def musicxml_to_notearray(
             notearray_or_part=unfolded_part,
             include_pitch_spelling=include_pitch_spelling,
             include_key_signature=include_key_signature,
-            include_time_signature=include_time_signature)
+            include_time_signature=include_time_signature,
+        )
         note_arrays.append(note_array)
 
     if len(note_arrays) == 1:
