@@ -1,4 +1,8 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
+"""
+Top level of the utilities module.
+"""
 
 from partitura.utils.generic import (
     ComparableMixin,
@@ -29,6 +33,7 @@ from partitura.utils.music import (
     ensure_pitch_spelling_format,
     ensure_notearray,
     compute_pianoroll,
+    compute_pitch_class_pianoroll,
     pianoroll_to_notearray,
     match_note_arrays,
     key_mode_to_int,
@@ -41,14 +46,37 @@ from partitura.utils.music import (
     update_note_ids_after_unfolding,
     note_name_to_pitch_spelling,
     note_name_to_midi_pitch,
-    pitch_spelling_to_note_name
+    pitch_spelling_to_note_name,
+    rest_array_from_part,
+    rest_array_from_rest_list,
+    ensure_rest_array,
+    rest_array_from_part_list,
+)
+from partitura.utils.synth import (
+    synthesize
 )
 
+from partitura.utils.misc import (
+    PathLike,
+    get_document_name,
+    deprecated_alias,
+    deprecated_parameter,
+)
+
+
 __all__ = [
+    "ensure_notearray",
+    "ensure_rest_array",
+    "compute_pianoroll",
+    "compute_pitch_class_pianoroll",
+    "pianoroll_to_notearray",
+    "slice_notearray_by_time",
     "key_name_to_fifths_mode",
     "fifths_mode_to_key_name",
     "key_mode_to_int",
     "pitch_spelling_to_midi_pitch",
-    "compute_pianoroll",
-    "pianoroll_to_notearray",
+    "pitch_spelling_to_note_name",
+    "show_diff",
+    "PrettyPrintTree",
+    "synthesize",
 ]
